import * as fs from 'fs/promises';
import * as path from 'path';
import { Injectable, Logger } from '@nestjs/common';
import { ChunkDto } from '@nice/common';

@Injectable()
export class ChunkManager {
    private readonly logger = new Logger(ChunkManager.name);

    private readonly CHUNK_PROCESSING_CONCURRENCY = 3;
    private readonly uploadDir: string;
    private readonly tempDir: string;

    constructor() {
        this.uploadDir = process.env.UPLOAD_DIR || path.join(process.cwd(), 'uploads');
        this.tempDir = path.join(this.uploadDir, 'temp');
        this.logger.localInstance.setLogLevels(["log", "debug"])
        this.logger.log(`Initialized ChunkManager with uploadDir: ${this.uploadDir}, tempDir: ${this.tempDir}`);
    }

    private validateChunk(chunk: ChunkDto, file: Express.Multer.File): void {
        this.logger.debug(`Validating chunk: identifier=${chunk?.identifier}, chunkNumber=${chunk?.chunkNumber}, bufferLength=${file?.buffer?.length}`);

        if (!chunk?.identifier || !chunk.chunkNumber || !file?.buffer?.length) {
            this.logger.warn('Invalid chunk metadata or buffer');
            throw new Error('Invalid chunk metadata or buffer');
        }
    }

    async save(chunk: ChunkDto, file: Express.Multer.File): Promise<void> {
        this.logger.log(`Saving chunk: identifier=${chunk.identifier}, chunkNumber=${chunk.chunkNumber}`);
        this.validateChunk(chunk, file);
        const chunkDir = path.join(this.tempDir, chunk.identifier);
        const chunkPath = path.join(chunkDir, `${chunk.chunkNumber}`);
        try {
            this.logger.debug(`Creating chunk directory: ${chunkDir}`);
            await fs.mkdir(chunkDir, { recursive: true, mode: 0o755 });

            this.logger.debug(`Writing chunk file: ${chunkPath}`);
            await fs.writeFile(chunkPath, file.buffer, { mode: 0o644 });

            this.logger.log(`Chunk saved successfully: ${chunkPath}`);
        } catch (error) {
            this.logger.error(`Chunk save failed: ${error instanceof Error ? error.message : error}`);
            throw error;
        }
    }

    async getChunks(identifier: string): Promise<number[]> {
        this.logger.log(`Retrieving chunks for identifier: ${identifier}`);
        if (!identifier) {
            this.logger.warn('No identifier provided');
            return [];
        }
        try {
            const chunkPath = path.join(this.tempDir, identifier);
            this.logger.debug(`Reading directory: ${chunkPath}`);

            const files = await fs.readdir(chunkPath);
            const chunks = files
                .map(Number)
                .filter(num => !isNaN(num) && num > 0)
                .sort((a, b) => a - b);

            this.logger.log(`Found chunks: ${chunks.join(', ')}`);
            return chunks;
        } catch (error) {
            this.logger.warn(`Chunk retrieval failed: ${error}`);
            return [];
        }
    }

    async merge(chunkDir: string, finalPath: string, totalChunks: number): Promise<void> {
        this.logger.log(`Merging chunks: chunkDir=${chunkDir}, finalPath=${finalPath}, totalChunks=${totalChunks}`);

        if (!chunkDir || !finalPath || totalChunks <= 0) {
            this.logger.warn('Invalid merge parameters');
            throw new Error('Invalid merge parameters');
        }

        try {
            this.logger.debug(`Opening final file: ${finalPath}`);
            const fileHandle = await fs.open(finalPath, 'w');

            try {
                for (let i = 0; i < totalChunks; i += this.CHUNK_PROCESSING_CONCURRENCY) {
                    const batch = Array.from(
                        { length: Math.min(this.CHUNK_PROCESSING_CONCURRENCY, totalChunks - i) },
                        (_, j) => i + j + 1
                    );
                    this.logger.debug(`Processing batch: ${batch.join(', ')}`);
                    const chunkBuffers = await Promise.all(
                        batch.map(chunkNumber => {
                            const chunkPath = path.join(chunkDir, `${chunkNumber}`);
                            this.logger.debug(`Reading chunk: ${chunkPath}`);
                            return fs.readFile(chunkPath);
                        })
                    );
                    for (const chunkBuffer of chunkBuffers) {
                        await fileHandle.write(chunkBuffer, 0, chunkBuffer.length);
                    }
                }
            } finally {
                this.logger.debug(`Closing file handle: ${finalPath}`);
                await fileHandle.close();
            }

            this.logger.log(`Merge completed successfully: ${finalPath}`);
        } catch (error) {
            this.logger.error(`Merge failed: ${error}`);
            throw error;
        }
    }

    async cleanup(identifier: string): Promise<void> {
        this.logger.log(`Cleaning up chunks for identifier: ${identifier}`);

        if (!identifier) {
            this.logger.warn('No identifier provided for cleanup');
            return;
        }

        try {
            const cleanupPath = path.join(this.tempDir, identifier);
            this.logger.debug(`Removing directory: ${cleanupPath}`);

            await fs.rm(cleanupPath, { recursive: true, force: true });
            this.logger.log(`Cleanup completed for: ${identifier}`);
        } catch (err) {
            if (err instanceof Error && err.name !== 'ENOENT') {
                this.logger.error(`Cleanup failed: ${err.message}`);
            }
        }
    }
}