import { Injectable, Logger } from '@nestjs/common';
import { Cron, CronExpression } from '@nestjs/schedule'; // 注意这里是 @Cron
import { UploadLock } from './types';
import { UploadLockInfo } from '@nice/common';

@Injectable()
export class UploadLockService {
    private readonly logger = new Logger(UploadLockService.name);
    private readonly uploadLocks: Map<string, UploadLock> = new Map();
    private readonly MAX_UPLOAD_DURATION = 2 * 60 * 60 * 1000; // 2 hours

    acquireLock(identifier: string, clientId: string): boolean {
        if (!identifier || !clientId) {
            this.logger.warn('Invalid lock parameters');
            return false;
        }
        const now = Date.now();
        const existingLock = this.uploadLocks.get(identifier);
        // Check if lock exists and is not expired
        if (existingLock) {
            const lockDuration = now - existingLock.timestamp;

            // More robust lock conflict check
            if (
                existingLock.clientId !== clientId &&
                lockDuration < this.MAX_UPLOAD_DURATION
            ) {
                this.logger.warn(`Upload conflict: File ${identifier} is locked by another client`);
                return false;
            }
        }

        // Acquire or update the lock
        this.uploadLocks.set(identifier, { clientId, timestamp: now });
        return true;
    }

    releaseLock(identifier: string, clientId: string): boolean {
        if (!identifier || !clientId) {
            this.logger.warn('Invalid unlock parameters');
            return false;
        }

        const currentLock = this.uploadLocks.get(identifier);

        // Only allow release by the current lock owner
        if (currentLock?.clientId === clientId) {
            this.uploadLocks.delete(identifier);
            return true;
        }

        this.logger.warn(`Unauthorized lock release attempt for ${identifier}`);
        return false;
    }

    checkLock(identifier: string): UploadLockInfo {
        const now = Date.now();
        const lockInfo = this.uploadLocks.get(identifier);
        // Check if lock exists and is not expired
        if (lockInfo && now - lockInfo.timestamp < this.MAX_UPLOAD_DURATION) {
            return {
                isLocked: true,
                lockedBy: lockInfo.clientId
            };
        }

        return { isLocked: false };
    }

    @Cron(CronExpression.EVERY_HOUR)
    cleanupExpiredLocks(): number {
        const now = Date.now();
        let removedLocksCount = 0;

        for (const [identifier, lock] of this.uploadLocks.entries()) {
            if (now - lock.timestamp > this.MAX_UPLOAD_DURATION) {
                this.uploadLocks.delete(identifier);
                removedLocksCount++;
            }
        }

        if (removedLocksCount > 0) {
            this.logger.log(`Cleaned up ${removedLocksCount} expired locks`);
        }

        return removedLocksCount;
    }
}