/**
 * 本模块定义了CancellationContext接口，用于管理请求的终止机制。
 * 该接口提供了两种请求终止方式：立即中止和优雅取消，适用于需要精细控制请求生命周期的场景。
 * 
 * 使用场景：
 * - 文件上传/下载过程中出现错误需要立即停止
 * - 用户主动取消长时间运行的请求
 * - 需要优雅关闭资源连接的场景
 */
export interface CancellationContext {
  /**
   * AbortSignal实例，用于监听取消请求事件。
   * 外部代码可以通过监听该信号来响应取消操作，实现资源的及时释放。
   * 
   * 设计模式：
   * - 采用观察者模式，允许外部订阅取消事件
   * - 与浏览器原生的AbortController API兼容
   */
  signal: AbortSignal

  /**
   * 立即中止请求的方法。
   * 该方法用于紧急情况下立即终止所有相关操作，适用于：
   * - 上传/下载过程中出现不可恢复的错误
   * - 检测到无效请求条件
   * - 需要立即释放系统资源的场景
   * 
   * 实现要求：
   * - 应立即停止所有相关操作
   * - 应释放所有已分配的资源
   * - 应触发signal的abort事件
   * 
   * 示例：
   * const context = createCancellationContext();
   * context.abort(); // 立即终止所有操作
   */
  abort: () => void

  /**
   * 优雅取消请求的方法。
   * 该方法允许请求在有限时间内完成收尾工作，适用于：
   * - 需要完成当前事务的场景
   * - 需要有序关闭资源的场景
   * - 需要确保数据一致性的场景
   * 
   * 实现要求：
   * - 应允许请求在合理时间内完成收尾工作
   * - 应在完成收尾后释放资源
   * - 应触发signal的abort事件
   * 
   * 示例：
   * const context = createCancellationContext();
   * context.cancel(); // 允许请求优雅结束
   * setTimeout(() => {
   *   if (!context.signal.aborted) {
   *     context.abort(); // 超时后强制中止
   *   }
   * }, 5000);
   */
  cancel: () => void
}
